@@ -4,6 +4,7 @@ Definition of the ApeGears ArgumentParser class.
 
 import argparse as _ap
 from collections import OrderedDict
+import inspect
 
 try:
     import argcomplete
@@ -12,6 +13,13 @@ except ImportError:
 
 from .misc import _ExtendAction, _SetItemAction, _KeyValueType, _StrictDefaultActionWrapper
 from .spec import find_spec as _find_spec
+from .lo99ing import add_log_levels_option
+
+
+################################################################################
+# Consts
+
+CALLER_DOC = ...
 
 
 ################################################################################
@@ -35,13 +43,33 @@ class ArgumentParser(_ap.ArgumentParser):
 
     ################################################################################
 
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
+    def __init__(self, *args, description=None, log_levels=None, **kwargs):
+        """
+        :param description:
+            if description=CALLER_DOC, will attempt to extract description from docstring of
+            caller module.
+        :param log_levels:
+            lo99ing integration.
+            a flag indicating whether to automatically add a log-level override option
+            (-L/--log-level).
+            if log_levels=None (default), will automatically add it only if the lo99ing package
+            is found.
+        """
+        # generate description:
+        if description is CALLER_DOC:
+            description = self._generate_descrption(stack_depth=1)
+
+        # call super:
+        super().__init__(*args, description=description, **kwargs)
 
         # register our actions
         self.register('action', 'extend', _ExtendAction)
         self.register('action', 'setitem', _SetItemAction)
 
+        # add default options
+        if log_levels or log_levels is None:
+            add_log_levels_option(self, force=bool(log_levels))
+
     ################################################################################
     # parse_args()
 
@@ -119,14 +147,7 @@ class ArgumentParser(_ap.ArgumentParser):
             Supports all kwargs supported by ``add_argument``, except for action and required.
         """
 
-        names = [name] if name else []
-        names, kwargs = self._use_spec(*names, is_positional=True, **kwargs)
-
-        if names and names[0] and names[0][0] in self.prefix_chars:
-            raise ValueError('name of positional must not start with %r' % self.prefix_chars)
-        for k in ['action', 'required']:
-            if k in kwargs:
-                raise ValueError('%s= does not apply to positionals' % k)
+        names, kwargs = self._process_positional(name, **kwargs)
 
         nargs = kwargs.pop('nargs', None)
         if 'default' in kwargs:
@@ -219,6 +240,35 @@ class ArgumentParser(_ap.ArgumentParser):
             **kwargs
         )
 
+    def add_positional_list(self, name=None, strict_default=True, **kwargs):
+        """
+        Add a *positional* list argument.  This calls ``add_argument`` with appropriate values.
+
+        :param kwargs:
+            Supports all kwargs supported by ``add_argument``, except for action and required.
+            nargs is typically not required.
+        :param strict_default: whether to enable workaround issue16399
+
+        :note: The default default value is an empty list.
+        """
+
+        names, kwargs = self._process_positional(name, **kwargs)
+
+        nargs = kwargs.pop('nargs', None)
+        if nargs is None:
+            nargs = '*'
+        if not isinstance(nargs, int) and nargs not in '*+':
+            raise ValueError(
+                'nargs=%s does not apply to a positional list' % nargs)
+
+        return self.add_argument(
+            *names,
+            action=None,
+            nargs=nargs,
+            strict_default=strict_default,
+            **kwargs
+        )
+
     def add_dict(self, *flags, key_type=None, key_metavar=None, strict_default=True, **kwargs):
         """
         Add an *optional* dict argument, where each item is of the form "KEY=VALUE".
@@ -408,6 +458,19 @@ class ArgumentParser(_ap.ArgumentParser):
             new_arg_value = post_process(arg_value)
             setattr(namespace, arg_name, new_arg_value)
 
+    def _process_positional(self, name, **kwargs):
+        names = [name] if name else []
+        names, kwargs = self._use_spec(*names, is_positional=True, **kwargs)
+
+        if names and names[0] and names[0][0] in self.prefix_chars:
+            raise ValueError('name of positional must not start with %r' % self.prefix_chars)
+
+        for k in ['action', 'required']:
+            if k in kwargs:
+                raise ValueError('%s= does not apply to positionals' % k)
+
+        return names, kwargs
+
     def _process_collection_optional(self, coll_cls, coll_cls_name, *flags, **kwargs):
 
         for k in ['action']:
@@ -481,5 +544,19 @@ class ArgumentParser(_ap.ArgumentParser):
                 return True
         return False
 
+    def _generate_descrption(self, stack_depth):
+        try:
+            stack = inspect.stack()
+            caller_frame = stack[stack_depth + 1]
+            raw_doc = caller_frame.frame.f_globals['__doc__'].strip()
+            lines = raw_doc.splitlines()
+            # return text up to the first blank line:
+            i = lines.index('')
+            if i >= 0:
+                lines = lines[:i]
+            return '\n'.join(lines)
+        except (KeyError, IndexError, ValueError, TypeError):
+            return None
+
 
 ################################################################################
